/***************************************************************************/
/*                                                                         */
/* servotogo.h                                                             */
/* This file contains driver functions for the ServoToGo card              */
/* Created by ServoToGo, Inc.                                              */
/* Heavily Modified 11-22-2002 by Tom Worsnopp                             */
/* Minorly Modified 01-22-2003 by Eric Faulring                            */
/*                                                                         */
/* These driver functions work for the ISA Model 2 ServoToGo I/O Boards    */
/*                                                                         */
/***************************************************************************/

#ifndef SERVOTOGO_H
#define SERVOTOGO_H
/***************************************************************************/
/*                                                                         */
/*                           #include files                                */
/*                                                                         */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/neutrino.h>
#include <sys/syspage.h>
#include <inttypes.h>
#include <sys/mman.h>
#include <hw/inout.h>
#include <math.h>

/***************************************************************************/
/*                                                                         */
/*                              #defines                                   */
/*                                                                         */
/***************************************************************************/

#define ADC_CHANNELS		0x03			// ADC channels 0-1, make this 0xFF for 0-7, 0x03 for 0-1, etc
#define NUM_STG_ADC_CHANNELS 2      // how many channels to you plan to convert
#define IRQ_NUMBER		3				// irq of servo-to-go
#define STGWMS_OVERHEAD 2600			// overhead of stgWaitMicroSeconds function call in terms of clock cycles (2600 for 1.533 GHz AMD Athlon)

#define MAX_AXIS			8				// number of encoder channels used - don't change this
#define STG_VOLTS_COUNT				0.002441406   // volts per count (20 volts for 2^13 counts)

#define TIMER12_RES					0.00000013968						// from page 24, Servo-to-Go manual
#define TIMER1_VALUE					2									// from page 24, Servo-to-Go manual (minimum value = 2, 180 results in freqency resolution of 254.9580155 Hz
																					// interrupt thread hangs if this is less than 180
#define TIMER0_RES					(TIMER1_VALUE*TIMER12_RES)		// calculation

#define CNT0_D     0x00
#define CNT0_C     0x02
#define CNT2_D     0x04
#define CNT4_D     0x08
#define CNT6_D     0x0c
#define CNT6_C     0x0e
#define DAC_0      0x10
#define ADC        0x410
#define CNTRL0     0x401
#define PORT_A     0x400
#define PORT_B     0x402
#define PORT_C     0x404
#define PORT_D     0x405
#define BRDTST     0x403
#define ABC_DIR    0x406
#define D_DIR      0x407
#define TIMER_0    0x408  // labeled backwards in the hardware manual
#define TIMER_1    0x40a  // labeled backwards in the hardware manual
#define TIMER_2    0x40c
#define TMRCMD     0x40e
#define IDLEN      0x409
#define SELDI      0x40b
#define IDL        0x40d
#define CNTRL1     0x40f

#define AUTOZERO       0x80
#define CALIBRATE_ADC  0x08
#define EOC            0x08

#define NOAUTOZERO          0x80
#define DONOTCALIBRATE_ADC  0x08

#define CNTRL1_WDTOUT    0x80
#define CNTRL1_INT_G2    0x40
#define CNTRL1_INT_T0    0x10
#define CNTRL1_INT_T2    0x20

/* hardware direction bit definitions */
#define A_DIR_BIT      0x10
#define B_DIR_BIT      0x02
#define C_HI_DIR_BIT   0x08
#define D_DIR_BIT      0x90
#define D_LOW_DIR_BIT  0x10
#define D_HI_DIR_BIT   0x80

/* digital input or output specifications - input by default*/
#define A_OUTPUT        0x10
#define A_INPUT         0x00
#define B_OUTPUT        0x02
#define B_INPUT         0x00
#define C_LOW_INPUT     0x00
#define C_LOW_OUTPUT    0x01
#define C_HI_INPUT      0x00
#define C_HI_OUTPUT     0x08
#define C_OUTPUT        (C_LOW_OUTPUT | C_HI_OUTPUT)
#define ALL_OUTPUT      (A_OUTPUT | B_OUTPUT | C_OUTPUT)  // not including port D
#define ALL_INPUT       0x00                              // not including port D
#define D_LOW_OUTPUT    0x01
#define D_HI_OUTPUT     0x08
#define D_OUTPUT        (D_LOW_OUTPUT | D_HI_OUTPUT)
#define D_INPUT         0X00

/* TMRCMD bits */
#define TMR_BIN    0x00   // binary - 16 bits
#define TMR_BCD    0x01   // BCD - 4 decades
#define TMR_M0     0x00   // mode 0 - interrupt on terminal count
#define TMR_M1     0x02   // mode 1 - programmable one-shot
#define TMR_M2     0x04   // mode 2 - real-time interrupt
#define TMR_M3     0x06   // mode 3 - square wave generator
#define TMR_M4     0x08   // mode 4 - software triggered strobe
#define TMR_M5     0x0a   // mode 5 - hardware triggered strobe
#define TMR_CL     0x00   // counter latch
#define TMR_LSB    0x10   // read/load LSB
#define TMR_MSB    0x20   // read/load MSB
#define TMR_LMSB   0X30   // read/load LSB followed by MSB
#define TMR_C0     0X00   // counter 0
#define TMR_C1     0X40   // counter 1
#define TMR_C2     0X80   // counter 2
#define TMR_RB     0Xc0   // read back

#define NO_BOARD       1
#define BOARD_PRESENT  0

#define NO_MODEL       0
#define MODEL2         2

#define CNTRL1_NOT_SLAVE 0x08



/***************************************************************************/
/*                                                                         */
/*                              typedefs                                   */
/*                                                                         */
/***************************************************************************/

typedef union{
  unsigned long               all;
  struct {unsigned char  A, B, C, D;} port;
} TYPE_IO32;

typedef union{
  long           Long;
  unsigned char  Byte[4];
} TYPE_LONGBYTE;

typedef struct ENC_TYPE{								/* defined here since belongs partly in cvt.h and partly in cyl.h */
   TYPE_LONGBYTE cnts[8];
   unsigned char index_latches;
} ENC_TYPE;

#ifdef __MAIN
#define EXTERN
#else
#define EXTERN extern
#endif

EXTERN	ENC_TYPE raw_STG_ENC_data;										/* used in cvt.h and cyl.h */
EXTERN   double raw_STG_ADC_data[8];

typedef union{
  unsigned short                          Word;
  struct   {unsigned char    high, low;}  Byte;
} TYPE_WORDBYTE;

/***************************************************************************/
/*                                                                         */
/*                           Locals                                        */
/*                                                                         */
/***************************************************************************/
#ifdef __STG
unsigned short  w_No_Board_Flag = NO_BOARD;
unsigned short  w_Model = NO_MODEL;
unsigned char   by_Old_Byte2[MAX_AXIS];
unsigned char   by_Enc_High_Byte[MAX_AXIS];
#endif

/***************************************************************************/
/*                                                                         */
/*                           Globals                                       */
/*                                                                         */
/***************************************************************************/
#ifdef __MAIN
struct sigevent Interrupt_Event;				/* tied to the board timers and our ISR */
unsigned short  stg_Base_Address = 0;		/* base address of servotogo board - set in function "stgInitialize()" */
int id;												/* thread id for interrupt attatch and detatch calls */
int stg_adc_timeout_errors = 0;				/* record error each time it occurs */
int stg_adc_timeout_printed_value;			/* so same error doesn't get printed multiple times */
double InterruptFrequency;						/* established based on "stgConfigureInterruptTimer(INT_FREQ)" */
uint64_t cps;										/* used for servotogo delay function */
unsigned short cntrl1;
#else
extern int stg_adc_timeout_errors;
extern int stg_adc_timeout_printed_value;
extern double InterruptFrequency;
extern unsigned short  stg_Base_Address;
extern uint64_t        cps;
extern unsigned short cntrl1;
#endif

/***************************************************************************/
/*                                                                         */
/*                         Function Prototypes                             */
/*                                                                         */
/***************************************************************************/

/* interrupt functions */
void            stgInitIRQ(unsigned short);
void            stgSetIRQ(unsigned short);
void            stgStopInterrupts();

/* DAC functions */
void            stgWriteDAC(unsigned short, long);

/* timer functions */
void            stgInitTimers();
void            stgSetTimer(unsigned short, unsigned char,unsigned short);
void            stgWaitMicroSeconds(int);
double			ConfigureTimers(double PeriodicFrequency);
double			stgConfigureTimerTwoFrequency(double TimerTwoFrequency);
double			stgConfigureInterruptTimer(double InterruptFrequency);

/* digital i/o (DIO) functions */
void            stgInitDIOs();
void            stgSetDioDirABC(unsigned char);
void            stgSetDioDirD(unsigned char);
unsigned long   stgReadDIOs();
unsigned char   stgReadDIO(unsigned char, unsigned short);
unsigned char   stgReadPortDIOs(unsigned short);
void            stgWriteDIO(unsigned char, unsigned char, unsigned short);
void            stgWriteDIOs(unsigned long);

/* index pulse functions */
void            stgSetIDLEN(unsigned char);
void            stgSetSELDI(unsigned char);
void            stgResetIDL();
unsigned char   stgGetIDL();

/* board functions */
void            stgInitVariables();
void            stgInitialize();
void            stgDisable();
void            stgInitBaseAddress();
unsigned short  stgBaseFind();
unsigned short  stgBrdtstOK(unsigned short);
unsigned short  stgGetModel();
unsigned short  stgGetBaseAddress();
void     fOutP(unsigned short, unsigned char);
void     fOutPW(unsigned short, unsigned short);
unsigned char   fInP(unsigned short);
unsigned short  fInPW(unsigned short);

/* ADC functions */
void            stgInitADCs();
double           stgReadADC(unsigned short);
void stgmReadADCs(unsigned char chan, double *values);

/* encoder functions */
void            stgInitEncoders();
void            stgInitializeIndexLatches();
void            stgLatchEncoders();
long            stgReadEncoder(unsigned short w_axis);
void            stgReadEncoders(TYPE_LONGBYTE *);
void            stgSetEncoderCounts(unsigned short, long);

#endif // SERVOTOGO_H
